'use strict';

/* eslint-disable no-unused-expressions */
/* eslint-disable no-undef */

const Q          = require('bluebird');
const chai       = require('chai');
const proxyquire = require('proxyquire');
const sinon      = require('sinon');
const sinonChai  = require('sinon-chai');
const request    = require('./../request');
const expect     = chai.expect;
chai.use(sinonChai);


describe('Cluster logic unit test', () => {

    function fakeAuthEntity() {
        return {
            _id: 'fake-id',
        }
    }

    function fakeOptionsObject() {
        return {};
    }

    const expectedHeadersEntries = ['X-Authenticated-Entity-Json'];

    function prepareProxy(opt) {
        return proxyquire('./../request', opt);
    }

    it('Should prepare request', () => {
        function mockPromiseRequest(opt) {
            expect(opt).to.not.be.empty;
            expect(opt.resolveWithFullResponse).to.be.true;
            expect(opt.json).to.be.true;
            expect(opt.headers).to.have.keys(expectedHeadersEntries);
            expect(opt.headers['X-Authenticated-Entity-Json']).to.be.deep.equal('eyJfaWQiOiJmYWtlLWlkIn0=');
            expect(JSON.parse(Buffer.from(opt.headers['X-Authenticated-Entity-Json'], 'base64').toString())).to.be.deep.equal(fakeAuthEntity());

            return Q.resolve({});
        }

        const requestProxy = prepareProxy({
            'request-promise': mockPromiseRequest
        });

        return requestProxy(fakeAuthEntity(), fakeOptionsObject());
    });

    it('Should throw an error on failed request', () => {

        function mockPromiseRequest() {
            return Q.reject({
                error: new Error('Native error should be wrapped'),
                message: 'Fake massage'
            });
        }

        const requestProxy = prepareProxy({
            'request-promise': mockPromiseRequest
        });

        return requestProxy(fakeAuthEntity(), fakeOptionsObject())
            .catch((err) => {
                expect(err).to.be.an.instanceOf(Error);
            });

    });

    it('Should return the body of the response', () => {
        function mockPromiseRequest() {
            return Q.resolve({
                body: {
                    key: 'val'
                }
            });
        }

        const requestProxy = prepareProxy({
            'request-promise': mockPromiseRequest
        });
        return requestProxy(fakeAuthEntity(), fakeOptionsObject())
            .then((body) => {
                expect(body).to.not.be.empty;
            });
    });

});